@@ -1,7 +1,7 @@
 /*
- * FCRON - periodic command scheduler 
+ * FCRON - periodic command scheduler
  *
- *  Copyright 2000-2021 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2025 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -12,11 +12,11 @@
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- * 
+ *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
+ *
  *  The GNU General Public License can also be found in the file
  *  `LICENSE' that comes with the fcron source distribution.
  */
@@ -25,8 +25,10 @@
 #include "fcron.h"
 
 #include "job.h"
+#include "mail.h"
 #include "temp_file.h"
 
+
 void sig_dfl(void);
 void end_job(cl_t * line, int status, FILE * mailf, short mailpos,
              char **sendmailenv);
@@ -44,8 +46,8 @@ void become_user(struct cl_t *cl, struct passwd *pas, char *home);
 
 #ifdef HAVE_LIBPAM
 void
-die_mail_pame(cl_t * cl, int pamerrno, struct passwd *pas, char *str,
-              env_list_t * env)
+die_mail_pame(cl_t *cl, int pamerrno, struct passwd *pas, char *str,
+              env_list_t *env)
 /* log an error in syslog, mail user if necessary, and die */
 {
     char buf[MAX_MSG];
@@ -276,9 +278,8 @@ sig_dfl(void)
     signal(SIGPIPE, SIG_DFL);
 }
 
-
 FILE *
-create_mail(cl_t * line, char *subject, char *content_type, char *encoding,
+create_mail(cl_t *line, char *subject, char *content_type, char *encoding,
             char **env)
     /* create a temp file and write in it a mail header */
 {
@@ -290,6 +291,7 @@ create_mail(cl_t * line, char *subject, char *content_type, char *encoding,
     /* hostname to add to email addresses (depending on if they have a '@') */
     char *hostname_from = "";
     char *hostname_to = "";
+    char *mailbox_addr = NULL;
     int i = 0;
 
     if (mailf == NULL)
@@ -318,8 +320,25 @@ create_mail(cl_t * line, char *subject, char *content_type, char *encoding,
     hostname[0] = '\0';
 #endif                          /* HAVE_GETHOSTNAME */
 
-    /* write mail header */
-    fprintf(mailf, "From: %s%s (fcron)\n", mailfrom, hostname_from);
+    /* write mail header. Global 'maildisplayname' comes from fcronconf.h */
+    if (maildisplayname[0] != '\0') {
+        /* New behavior -- RFC-compliant */
+        mailbox_addr =
+            make_mailbox_addr(maildisplayname, mailfrom, hostname_from);
+        if (!mailbox_addr) {
+            warn("could not make the mailbox address");
+        }
+    }
+    if (mailbox_addr) {
+        fprintf(mailf, FROM_HEADER_KEY "%s\n", mailbox_addr);
+        Free_safe(mailbox_addr);
+    }
+    else {
+        /* Old behavior */
+        fprintf(mailf, FROM_HEADER_KEY "%s%s (fcron)\n", mailfrom,
+                hostname_from);
+    }
+
     fprintf(mailf, "To: %s%s\n", line->cl_mailto, hostname_to);
 
     if (subject)
@@ -447,7 +466,7 @@ read_write_pipe(int fd, void *buf, size_t size, int action)
 
 int
 read_pipe(int fd, void *buf, size_t size)
-    /* Read data from pipe. 
+    /* Read data from pipe.
      * Handles signal interruptions, and read in several passes.
      * Returns ERR in case of a closed pipe, the errno from read
      * for other errors, and OK if everything was read successfully */
@@ -457,7 +476,7 @@ read_pipe(int fd, void *buf, size_t size)
 
 int
 write_pipe(int fd, void *buf, size_t size)
-    /* Read data from pipe. 
+    /* Read data from pipe.
      * Handles signal interruptions, and read in several passes.
      * Returns ERR in case of a closed pipe, the errno from write
      * for other errors, and OK if everything was read successfully */
@@ -466,7 +485,7 @@ write_pipe(int fd, void *buf, size_t size)
 }
 
 void
-run_job_grand_child_setup_stderr_stdout(cl_t * line, int *pipe_fd)
+run_job_grand_child_setup_stderr_stdout(cl_t *line, int *pipe_fd)
     /* setup stderr and stdout correctly so as the mail containing
      * the output of the job can be send at the end of the job.
      * Close the pipe (both ways). */
@@ -501,7 +520,7 @@ run_job_grand_child_setup_stderr_stdout(cl_t * line, int *pipe_fd)
 }
 
 void
-run_job_grand_child_setup_nice(cl_t * line)
+run_job_grand_child_setup_nice(cl_t *line)
     /* set the nice value for the job */
 {
     if (line->cl_nice != 0) {
@@ -711,7 +730,7 @@ run_job(struct exe_t *exeent)
                 xclose_check(&(pipe_pid_fd[1]), "child's pipe_pid_fd[1]");
 
                 /* we use a while because of a possible interruption by a signal */
-                while ((pid = wait3(&status, 0, NULL)) > 0) {
+                while ((pid = waitpid(-1, &status, 0)) > 0) {
 #ifdef CHECKRUNJOB
                     debug("run_job(): child: ending job pid %d", pid);
 #endif                          /* CHECKRUNJOB */
@@ -766,8 +785,7 @@ run_job(struct exe_t *exeent)
 }
 
 void
-end_job(cl_t * line, int status, FILE * mailf, short mailpos,
-        char **sendmailenv)
+end_job(cl_t *line, int status, FILE *mailf, short mailpos, char **sendmailenv)
     /* if task have made some output, mail it to user */
 {
 
@@ -853,7 +871,7 @@ end_job(cl_t * line, int status, FILE * mailf, short mailpos,
 }
 
 void
-launch_mailer(cl_t * line, FILE * mailf, char **sendmailenv)
+launch_mailer(cl_t *line, FILE *mailf, char **sendmailenv)
     /* mail the output of a job to user */
 {
 #ifdef USE_SENDMAIL
